/* 
 * Copyright (c) 1995-1994 The University of Utah and
 * the Computer Systems Laboratory at the University of Utah (CSL).
 * All rights reserved.
 *
 * Permission to use, copy, modify and distribute this software is hereby
 * granted provided that (1) source code retains these copyright, permission,
 * and disclaimer notices, and (2) redistributions including binaries
 * reproduce the notices in supporting documentation, and (3) all advertising
 * materials mentioning features or use of this software display the following
 * acknowledgement: ``This product includes software developed by the
 * Computer Systems Laboratory at the University of Utah.''
 *
 * THE UNIVERSITY OF UTAH AND CSL ALLOW FREE USE OF THIS SOFTWARE IN ITS "AS
 * IS" CONDITION.  THE UNIVERSITY OF UTAH AND CSL DISCLAIM ANY LIABILITY OF
 * ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * CSL requests users of this software to return to csl-dist@cs.utah.edu any
 * improvements that they make and grant CSL redistribution rights.
 *
 *      Author: Bryan Ford, University of Utah CSL
 */

#include <sys/types.h>
#include <fcntl.h>
#include <errno.h>

#include "dos_io.h"
#include "debug.h"

int dos_open(const char *s, int flags, int mode, dos_fd_t *out_fh)
{
	struct real_call_data real_call_data;
	int err = EINVAL; /*XXX*/

	dos_init_rcd(&real_call_data);

	if (flags & O_APPEND)
		return EINVAL;

	if (strlen(s)+1 > DOS_BUF_SIZE)
		return E2BIG;
	strcpy(dos_buf, s);
	real_set_ds_dx(dos_buf);

	if (!(flags & O_TRUNC))
	{
		real_call_data.eax = 0x3d00 | (flags & O_ACCMODE);
		real_call_data.ecx = 0;
		real_int(0x21, &real_call_data);
		if (!(err = dos_check_err(&real_call_data)))
		{
			*out_fh = real_call_data.eax & 0xffff;
			return 0;
		}
	}

	if (flags & O_CREAT)
	{
		real_call_data.eax = 0x3c00;
		real_call_data.ecx = mode;
		real_int(0x21, &real_call_data);
		if (!(err = dos_check_err(&real_call_data)))
		{
			*out_fh = real_call_data.eax & 0xffff;
			return 0;
		}
	}

	return err;
}

