/*
 * Interface Linux drivers to Mach and vice versa.
 * Copyright (C) 1995 Shantanu Goel.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * XXX: The routines in this file are presently geared towards Ethernet.
 */

#define LINUX_MACH_INCLUDE

#include <sys/types.h>

#include <mach/vm_param.h>

#include <kern/zalloc.h>
#include <kern/kalloc.h>

#include <vm/vm_kern.h>
#include <vm/vm_object.h>
#include <vm/vm_map.h>

#include <device/device_types.h>
#include <device/errno.h>
#include <device/io_req.h>
#include <device/if_hdr.h>
#include <device/if_ether.h>
#include <device/net_status.h>
#include <device/net_io.h>

#include <i386/ipl.h>
#include <i386/pic.h>

#include <linux/config.h>
#include <linux/interrupt.h>
#include <linux/skbuff.h>
#include <linux/netdevice.h>
#include <linux/mm.h>
#include <linux/malloc.h>

#include <asm/system.h>
#define size_t	unsigned
#include <asm/string.h>
#undef size_t

#define bzero(s, c)	memset(s, 0, c)
#define bcopy(s, d, c)	memcpy(d, s, c)

/*
 * Flag to indicate a device uses DMA.
 */
#define IFF_DMA		0x8000

/*
 * List of network devices found.
 */
static struct linux_device *net_devs = 0;

/*
 * Used during configuration to assign device unit numbers.
 */
static int dev_unit;

/*
 * Used during configuration to assign the name
 * by which Mach refers to a device.
 */
static char *dev_name;

/*
 * Forward declarations.
 */
void linux_ether_setup(struct linux_device *);
static void net_soft(void *);
static void dev_probe(char *, int (*)(struct linux_device *), int);
static void dev_init(char *, int (*)(void));

extern int linux_intr_pri;
extern int linux_request_dma_called;

/*
 * Probe for multiple instances of a device.
 */
#define PROBE_DEV(name, probe) {			\
	extern int probe(struct linux_device *);	\
	dev_probe(name, probe, 0); 			\
}

/*
 * Probe for a single instance of a device.
 */
#define PROBE_DEV_ONCE(name, probe) {			\
	extern int probe(struct linux_device *);	\
	dev_probe(name, probe, 1);			\
}

/*
 * Like PROBE_DEV() but for a driver that needs to allocate DMA memory.
 */
#define INIT_DEV(name, init) {	\
	extern int init(void);	\
	dev_init(name, init);	\
}

/*
 * Initialize network devices.
 */
void
linux_net_dev_init()
{
	/*
	 * Install software interrupt handler.
	 */
	linux_bh_base[NET_BH].routine = net_soft;
	linux_bh_base[NET_BH].data = 0;
	enable_bh(NET_BH);

	/*
	 * Set interrupt priority in case a driver calls request_irq().
	 */
	linux_intr_pri = SPL6;

	/*
	 * Probe and configure devices.
	 */

#ifdef CONFIG_LANCE			/* AMD 7990 LANCE */
	INIT_DEV("le", lance_init)
#endif

#ifdef CONFIG_DEC_ELCP			/* DEC 21040 */
	INIT_DEV("dt", dec21040_init)
#endif

#ifdef CONFIG_SEEQ8005
        PROBE_DEV("sq", seeq8005_probe);
#endif

#ifdef CONFIG_HP100
	PROVE_DEV("ho", hp100_probe);
#endif

#ifdef CONFIG_ULTRA			/* SMC Ultra */
	PROBE_DEV("ul", ultra_probe);
#endif

#ifdef CONFIG_WD80x3			/* Western Digital/SMC boards */
	PROBE_DEV("wd", wd_probe);
#endif

#ifdef CONFIG_EL2			/* 3c503 */
	PROBE_DEV("et", el2_probe);
#endif

#ifdef CONFIG_NE2000			/* Novell NE2000 */
	PROBE_DEV("ne", ne_probe);
#endif

#ifdef CONFIG_HPLAN
	PROBE_DEV("hl", hp_probe);
#endif

#ifdef CONFIG_HPLAN_PLUS
	PROBE_DEV("hp", hp_plus_probe);
#endif

#ifdef CONFIG_AT1700
	PROBE_DEV("as", at1700_probe);
#endif

#ifdef CONFIG_EL3			/* 3c509 */
	PROBE_DEV("eh", el3_probe);
#endif

#ifdef CONFIG_ZNET			/* Zenith Z-Note, some IBM thinkpads */
	PROBE_DEV_ONCE("zn", znet_probe);
#endif

#ifdef CONFIG_EEXPRESS			/* Intel EtherExpress */
	PROBE_DEV("ex", express_probe);
#endif

#ifdef CONFIG_EEXPRESS_PRO
	PROBE_DEV("er", eepro_probe);
#endif

#ifdef CONFIG_DEPCA			/* DEC DEPCA */
	PROBE_DEV("dp", depca_probe);
#endif

#ifdef CONFIG_EWRK3			/* DEC EtherWORKS 3 */
	PROBE_DEV("ew", ewrk3_probe);
#endif

#ifdef CONFIG_DE4X5			/* DEC DE425, DE434, DE435 */
	PROBE_DEV("df", de4x5_probe);
#endif

#ifdef CONFIG_APRICOT			/* Apricot I82596 */
	PROBE_DEV("ap", apricot_probe);
#endif

#ifdef CONFIG_EL1			/* 3c501 */
	PROBE_DEV("el", el1_probe);
#endif

#ifdef CONFIG_WAVELAN			/* NCR/AT&T WaveLAN (radio Ethernet) */
	PROBE_DEV("wl", wavelan_probe);
#endif

#ifdef CONFIG_EL16			/* 3c507 */
	PROBE_DEV("es", el16_probe);
#endif

#ifdef CONFIG_ELPLUS			/* 3c505 */
	PROBE_DEV("ep", elplus_probe);
#endif

#ifdef CONFIG_AC3200			/* Ansel Communications EISA 3200 */
	PROBE_DEV("ac", ac3200_probe);
#endif

#ifdef CONFIG_E2100			/* Cabletron E21xx series */
	PROBE_DEV("cb", e2100_probe);
#endif

#ifdef CONFIG_DE600			/* D-Link DE-600 adapter */
	PROBE_DEV_ONCE("dl", de600_probe);
#endif

#ifdef CONFIG_DE620			/* D-Link DE-620 adapter */
	PROBE_DEV_ONCE("de", de620_probe);
#endif

#ifdef CONFIG_SK_G16
	PROBE_DEV("sk", SK_init);
#endif

#ifdef CONFIG_NI52
	PROBE_DEV("nf", ni52_probe);
#endif

#ifdef CONFIG_NI65
	PROBE_DEV("ns", ni65_probe);
#endif

#ifdef CONFIG_ATP			/* RealTek pocket ethernet adaptor */
	PROBE_DEV_ONCE("rt", atp_init);
#endif

	/*
	 * Allocate and initialize Mach data for each device found.
	 */
	if (net_devs) {
		int i;
		struct linux_device *dev;
		struct ifnet *ifp;

		for (dev = net_devs, i = 0; dev; dev = dev->next, i++)
			;
		ifp = (struct ifnet *)kalloc(sizeof(struct ifnet) * i);
		if (!ifp)
			panic("linux_net_dev_init: unable to alloc mach data");
		for (dev = net_devs; dev; dev = dev->next) {
			int dma;

			dma = dev->mach_data ? IFF_DMA : 0;
			dev->mach_data = ifp;
			ifp->if_unit = dev->name[strlen(dev->name) - 1] - '0';
			ifp->if_mtu = dev->mtu;
			ifp->if_flags = dev->flags | dma;
			ifp->if_header_size = dev->hard_header_len;
			ifp->if_header_format = dev->type;
			ifp->if_address_size = dev->addr_len;
			ifp->if_address = dev->dev_addr;
			if_init_queues(ifp);
			ifp++;
		}
	}
}

/*
 * Probe for a device.
 */
static void
dev_probe(name, probe, once)
	char *name;
	int (*probe)(struct linux_device *);
	int once;
{
	int amt, i;
	struct linux_device *dev, *devp;

	/*
	 * Set device name and unit number for linux_init_etherdev().
	 */
	dev_name = name;
	dev_unit = 0;

	/*
	 * Amount of space to allocate for device structure.
	 */
	amt = sizeof(*dev) + strlen(name) + 2;

	while (1) {
		/*
		 * Allocate and initialize device structure.
		 */
		dev = (struct linux_device *)kalloc(amt);
		if (!dev)
			panic("dev_probe: unable to alloc dev");
		bzero(dev, amt);
		dev->name = (char *)(dev + 1);
		sprintf(dev->name, "%s%d", dev_name, dev_unit);

		linux_request_dma_called = 0;

		/*
		 * Call the probe routine.
		 * Terminate loop if device does not exist.
		 */
		if ((*probe)(dev)) {
			kfree((vm_offset_t)dev, amt);
			break;
		}

		/*
		 * Find out how many devices the driver found.
		 * Some Linux drivers probe for all instances
		 * of a device in one shot.
		 */
		for (devp = dev, i = 0; devp->next; devp = devp->next, i++)
			;

		/*
		 * Reset device names in case the driver mucked
		 * with them.
		 */
		if (i == 0) {
			sprintf(dev->name, "%s%d", dev_name, dev_unit);
			if (linux_request_dma_called)
				dev->mach_data = (void *)2;
		} else {
			for (devp = dev, i = 0;; devp = devp->next, i++) {
				sprintf(devp->name, "%s%d", dev_name, i);
				if (linux_request_dma_called)
					devp->mach_data = (void *)2;
				if (!devp->next)
					break;
			}
		}

		/*
		 * Add device(s) to list.
		 */
		devp->next = net_devs;
		net_devs = dev;

		/*
		 * Terminate if the driver has already probed
		 * for multiple instances of the device, or
		 * if only one instance of the device is to be probed.
		 */
		if (i || once)
			break;

		dev_unit++;
	}
}

/*
 * Probe for a device that uses DMA.
 */
static void
dev_init(name, init)
	char *name;
	int (*init)(void);
{
	/*
	 * Set device name and unit number for linux_init_etherdev().
	 */
	dev_name = name;
	dev_unit = 0;

	linux_request_dma_called = 0;

	/*
	 * Call initialization routine.
	 */
	(*init)();

	if (linux_request_dma_called) {
		int len = strlen(name);
		struct linux_device *dev = net_devs;

		while (dev && !strncmp(dev->name, name, len)) {
			dev->mach_data = (void *)2;
			dev = dev->next;
		}
	}
}

/*
 * Called by a driver on a successful probe
 * to allocate a `struct linux_device'.
 * For Linux emulation, a driver will call
 * this routine if it uses DMA or to allocate
 * private driver data.
 */
struct linux_device *
linux_init_etherdev(dev, sizeof_private)
	struct linux_device *dev;
	int sizeof_private;
{
	/*
	 * Allocate device structure and add it to the
	 * device list, if required.
	 */
	if (!dev) {
		int amt;

		amt = sizeof(*dev) + strlen(dev_name) + 2;
		dev = (struct linux_device *)kalloc(amt);
		if (!dev)
			panic("linux_init_etherdev: "
			      "could not allocate dev structure");
		bzero(dev, amt);
		dev->name = (char *)(dev + 1);
		sprintf(dev->name, "%s%d", dev_name, dev_unit);
		dev->next = net_devs;
		net_devs = dev;
		dev_unit++;
	}

	/*
	 * Allocate private driver data.
	 */
	if (sizeof_private) {
		dev->priv = linux_kmalloc(sizeof_private, GFP_KERNEL);
		if (!dev->priv)
			panic("linux_init_etherdev: "
			      "could not allocate private driver data");
		bzero(dev->priv, sizeof_private);
	} else
		dev->priv = 0;

	linux_ether_setup(dev);

	return (dev);
}

/*
 * Called by an Ethernet driver to initialize
 * sizes of headers, etc.
 */
void
linux_ether_setup(dev)
	struct linux_device *dev;
{
	dev->flags = IFF_BROADCAST;
	dev->mtu = ETHERMTU;
	dev->hard_header_len = sizeof(struct ether_header);
	dev->type = HDR_ETHERNET;
	dev->addr_len = 6;
}

/*
 * Allocate an sk_buff.
 */
struct sk_buff *
linux_dev_alloc_skb(size)
	unsigned int size;
{
	struct sk_buff *skb;

	skb = linux_kmalloc(size + sizeof(*skb), GFP_KERNEL);
	if (skb) {
		skb->len = size;
		skb->mach_len = sizeof(*skb) + size;
		skb->data = (unsigned char *)(size ? skb + 1 : 0);
		skb->mach_data = 0;
	}
	return (skb);
}

/*
 * Deallocate an skbuff.
 */
void
linux_dev_kfree_skb(skb, mode)
	struct sk_buff *skb;
	int mode;
{
	int len = skb->len;
	io_req_t ior = skb->mach_data;

	linux_kfree_s(skb, skb->mach_len);

	if (ior)
		iodone(ior);
}

/*
 * Called by a driver to deliver a packet received on an interface.
 */
void
linux_netif_rx(skb)
	struct sk_buff *skb;
{
	int len = skb->len;
	ipc_kmsg_t kmsg;
	struct ether_header *eh;
	struct packet_header *pkt;
	struct linux_device *dev = skb->dev;
	struct ifnet *ifp = dev->mach_data;

	/*
	 * Grab a message buffer.
	 */
	kmsg = net_kmsg_get();
	if (kmsg == 0) {
		linux_dev_kfree_skb(skb, 0);
		return;
	}

	/*
	 * Copy data into message buffer.
	 */
	eh = (struct ether_header *)(net_kmsg(kmsg)->header);
	pkt = (struct packet_header *)(net_kmsg(kmsg)->packet);
	bcopy(skb->data, eh, sizeof(*eh));

	len -= sizeof(*eh);
	bcopy(skb->data + sizeof(*eh), pkt + 1, len);

	/*
	 * Free the sk_buff.
	 */
	linux_dev_kfree_skb(skb, 0);

	/*
	 * Pass packet up to the microkernel.
	 */
	pkt->type = eh->ether_type;
	pkt->length = len + sizeof(*pkt);
	net_packet(ifp, kmsg, pkt->length, ethernet_priority(kmsg));
}

/*
 * Old interface to netif_rx().
 */
int
linux_dev_rint(buf, size, flags, dev)
	unsigned char *buf;
	long size;
	int flags;
	struct linux_device *dev;
{
	struct sk_buff *skb = (struct sk_buff *)buf;

	skb->dev = dev;
	skb->len = size;
	linux_netif_rx(skb);
	return (0);
}

/*
 * Software interrupt handler.
 * Start transmission on interfaces.
 */
static void
net_soft(xxx)
	void *xxx;
{
	unsigned bufsize;
	vm_offset_t pa;
	io_req_t ior;
	struct linux_device *dev;
	struct sk_buff *skb;
	struct ifnet *ifp;

	for (dev = net_devs; dev; dev = dev->next) {
		ifp = dev->mach_data;
		while (1) {
			IF_DEQUEUE(&ifp->if_snd, ior);
			if (!ior)
				break;
			bufsize = 0;
			if (ifp->if_flags & IFF_DMA) {
				pa = pmap_extract(kernel_pmap,
						  (vm_offset_t)ior->io_data);
				if (ior->io_count
				    > (I386_PGBYTES
				       - (pa & (I386_PGBYTES - 1))))
					bufsize = ior->io_count;
			}
			skb = linux_dev_alloc_skb(bufsize);
			if (!skb) {
				IF_PREPEND(&ifp->if_snd, ior);
				printf("net_soft: skb malloc failed\n");
				mark_bh(NET_BH);
				break;
			}
			if (bufsize)
				bcopy(ior->io_data, skb->data, ior->io_count);
			else if (ifp->if_flags & IFF_DMA)
				skb->data = (unsigned char *)pa;
			else
				skb->data = (unsigned char *)ior->io_data;
			skb->len = ior->io_count;
			skb->dev = dev;
			skb->mach_data = ior;
			if ((*dev->hard_start_xmit)(skb, dev) != 0) {
				skb->mach_data = 0;
				linux_dev_kfree_skb(skb, 0);
				IF_PREPEND(&ifp->if_snd, ior);
				mark_bh(NET_BH);
				break;
			}
		}
	}
}

/*
 * Given a device name and unit number, find it's
 * Linux device structure.
 */
static inline struct linux_device *
name2dev(name, unit)
	char *name;
	int unit;
{
	char fullname[50];
	struct linux_device *dev;

	sprintf(fullname, "%s%d", name, unit);
	for (dev = net_devs; dev; dev = dev->next)
		if (!strcmp(fullname, dev->name))
			return (dev);
	return (NULL);
}

/*
 * Mach kernel interface routines.
 */

/*
 * Open an interface.
 */
int
linux_net_open(unit, flag, ior)
	int unit, flag;
	io_req_t ior;
{
	struct linux_device *dev;
	struct ifnet *ifp;

	/*
	 * Set priority at which the device will interrupt
	 * for linux_request_irq().
	 */
	linux_intr_pri = SPL6;
	linux_request_dma_called = 0;

	/*
	 * Locate device and call it's open routine.
	 */
	dev = name2dev(ior->io_device->dev_ops->d_name, unit);
	if (dev == NULL || dev->open == 0 || (*dev->open)(dev) != 0)
		return (D_NO_SUCH_DEVICE);

	/*
	 * Mark interface as up.
	 */
	ifp = dev->mach_data;
	ifp->if_flags |= IFF_UP|IFF_RUNNING;
	if (linux_request_dma_called)
		ifp->if_flags |= IFF_DMA;

	return (0);
}

/*
 * Output data on an interface.
 */
int
linux_net_write(unit, ior)
	int unit;
	io_req_t ior;
{
	int rc, s;
	boolean_t wait;
	struct linux_device *dev;
	struct ifnet *ifp;

	dev = name2dev(ior->io_device->dev_ops->d_name, unit);
	if (dev == NULL)
		return (D_NO_SUCH_DEVICE);

	ifp = dev->mach_data;

	/*
	 * Reject the write if the interface is down.
	 */
	if ((ifp->if_flags & (IFF_UP|IFF_RUNNING)) != (IFF_UP|IFF_RUNNING))
		return (D_DEVICE_DOWN);

	/*
	 * Reject the write if the packet is too large or too small.
	 */
	if (ior->io_count < ifp->if_header_size
	    || ior->io_count > ifp->if_header_size + ifp->if_mtu)
		return (D_INVALID_SIZE);

	/*
	 * Wire down the memory.
	 */
	rc = device_write_get(ior, &wait);
	if (rc != KERN_SUCCESS)
		return (rc);

	/*
	 * Network interfaces can't cope with VM continuations.
	 * If wait is set, just panic.
	 */
	if (wait)
		panic("linux_net_write: VM continuation");

	/*
	 * Enqueue packet and enable software interrupt.
	 */
	s = spl1();
	IF_ENQUEUE(&ifp->if_snd, ior);
	mark_bh(NET_BH);
	splx(s);

	return (D_IO_QUEUED);
}

/*
 * Get device status.
 */
int
linux_net_getstat(mach_dev, flavor, status, count)
	struct device *mach_dev;
	int flavor;
	dev_status_t status;
	unsigned *count;
{
	struct linux_device *dev;

	dev = name2dev(mach_dev->dev_ops->d_name, mach_dev->dev_number);
	if (dev == NULL)
		return (D_NO_SUCH_DEVICE);

	return (net_getstat(dev->mach_data, flavor, status, count));
}

/*
 * Set device status.
 */
int
linux_net_setstat(mach_dev, flavor, status, count)
	struct device *mach_dev;
	int flavor;
	dev_status_t status;
	unsigned count;
{
	int oflags;
	struct ifnet *ifp;
	struct net_status *ns;
	struct linux_device *dev;

	dev = name2dev(mach_dev->dev_ops->d_name, mach_dev->dev_number);
	if (dev == NULL)
		return (D_NO_SUCH_DEVICE);

	ifp = dev->mach_data;

	switch (flavor) {

	case NET_STATUS:
		if (count < NET_STATUS_COUNT)
			return (D_INVALID_SIZE);
		ns = (struct net_status *)status;
		oflags = ifp->if_flags & (IFF_ALLMULTI|IFF_PROMISC);
		ifp->if_flags &= ~(IFF_ALLMULTI|IFF_PROMISC);
		ifp->if_flags |= ns->flags & (IFF_ALLMULTI|IFF_PROMISC);
		if ((ifp->if_flags & (IFF_ALLMULTI|IFF_PROMISC)) != oflags) {
			/*
			 * XXX: need to support multicast.
			 */
			if (ifp->if_flags & IFF_PROMISC)
				(*dev->set_multicast_list)(dev, -1, NULL);
			else
				(*dev->set_multicast_list)(dev, 0, NULL);
		}
		return (0);

	default:
		return (D_INVALID_OPERATION);
	}
}

/*
 * Install a packet filter.
 */
int
linux_net_setinput(mach_dev, receive_port, priority, filter, filter_count)
	struct device *mach_dev;
	mach_port_t receive_port;
	int priority;
	filter_t *filter;
	unsigned filter_count;
{
	struct linux_device *dev;

	dev = name2dev(mach_dev->dev_ops->d_name, mach_dev->dev_number);
	if (dev == NULL)
		return (D_NO_SUCH_DEVICE);

	return (net_set_filter(dev->mach_data, receive_port,
			       priority, filter, filter_count));
}
