/*
 * Linux timers.
 * Copyright (C) 1995 Shantanu Goel.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *  linux/kernel/sched.c
 *
 *  Copyright (C) 1991, 1992  Linus Torvalds
 */

#define LINUX_MACH_INCLUDE

#include <sys/types.h>
#include <kern/time_out.h>
#include <linux/timer.h>
#include <asm/system.h>

/*
 * Mask of active timers.
 */
unsigned long linux_timer_active = 0;

/*
 * List of timeout routines.
 */
struct linux_timer_struct linux_timer_table[32];


/*
 * Timer software interrupt handler.
 */
void
linux_timer_soft()
{
	unsigned long mask;
	struct linux_timer_struct *tp;

	for (mask = 1, tp = linux_timer_table; mask; tp++, mask <<= 1) {
		if (mask > linux_timer_active)
			break;
		if ((mask & linux_timer_active)
		    && tp->expires > elapsed_ticks) {
			linux_timer_active &= ~mask;
			(*tp->fn)();
			sti();
		}
	}
}

/*
 * Schedule a timeout routine.
 */
void
linux_add_timer(timer)
	struct linux_timer_list *timer;
{
	if (timer->next || timer->prev) {
		printf("linux_add_timer() called with non-zero list from %p\n",
		       __builtin_return_address(0));
		return;
	}
	timeout(timer->function, timer->data, timer->expires);
}

/*
 * Unschedule a timeout routine.
 */
int
linux_del_timer(timer)
	struct linux_timer_list *timer;
{
	return (untimeout(timer->function, timer->data));
}
